import 'package:article_flutter_riverpod/article_02/consumer_example.dart';
import 'package:article_flutter_riverpod/article_02/parent_example.dart';
import 'package:article_flutter_riverpod/article_02/stateful_example.dart';
import 'package:article_flutter_riverpod/article_02/stateless_example.dart';
import 'package:article_flutter_riverpod/example.dart';
import 'package:article_flutter_riverpod/presentation/01_introduction_example.dart' as introduction_example;
import 'package:article_flutter_riverpod/presentation/02_provider_example.dart' as provider_example;
import 'package:article_flutter_riverpod/presentation/03_select_example.dart' as select_example;
import 'package:article_flutter_riverpod/presentation/04_stateful_example.dart' as stateful_example;
import 'package:article_flutter_riverpod/presentation/05_state_provider_example.dart' as state_provider_example;
import 'package:article_flutter_riverpod/presentation/07_overrides_example.dart' as overrides_example;
import 'package:article_flutter_riverpod/presentation/08_future_provider_example.dart' as future_provider_example;
import 'package:article_flutter_riverpod/presentation/09_async_value_example.dart' as async_value_example;
import 'package:flutter/material.dart';
import 'package:go_router/go_router.dart';
import 'package:sticky_headers/sticky_headers.dart';

final examples = [
  ExampleGroup(
    title: 'Article 01',
    examples: [
      Example(
        title: 'Consumer',
        icon: Icons.eco,
        path: 'consumer',
        builder: (_) => const ConsumerExample(),
      ),
      Example(
        title: 'Stateless',
        icon: Icons.remove_circle_outline,
        path: 'stateless',
        builder: (_) => const StatelessExample(),
      ),
      Example(
        title: 'Stateful',
        icon: Icons.add_circle_outline,
        path: 'stateful',
        builder: (_) => const StatefulExample(),
      ),
      Example(
        title: 'Parent ProviderScope',
        icon: Icons.child_care,
        path: 'parent-providerscope',
        builder: (_) => const ParentExample(),
      ),
    ],
  ),
  ExampleGroup(
    title: 'Presentation',
    examples: [
      Example(
        title: 'Introduction',
        icon: Icons.eco,
        path: 'todo-introduction',
        builder: (_) => const introduction_example.TodoExample(),
        isScrollable: false,
      ),
      Example(
        title: 'Provider',
        icon: Icons.arrow_forward,
        path: 'todo-provider',
        builder: (_) => const provider_example.TodoExample(),
        isScrollable: false,
      ),
      Example(
        title: 'Select',
        icon: Icons.filter_alt,
        path: 'todo-select',
        builder: (_) => const select_example.TodoExample(),
        isScrollable: false,
      ),
      Example(
        title: 'Stateful',
        icon: Icons.add_circle_outline,
        path: 'todo-stateful',
        builder: (_) => const stateful_example.TodoExample(),
        isScrollable: false,
      ),
      Example(
        title: 'StateProvider',
        icon: Icons.folder_open,
        path: 'todo-state-provider',
        builder: (_) => const state_provider_example.TodoExample(),
        isScrollable: false,
      ),
      Example(
        title: 'Overrides',
        icon: Icons.filter,
        path: 'todo-overrides',
        builder: (_) => const overrides_example.TodoExample(),
        isScrollable: false,
      ),
      Example(
        title: 'FutureProvider',
        icon: Icons.refresh,
        path: 'todo-future-provider',
        builder: (_) => const future_provider_example.TodoExample(),
        isScrollable: false,
      ),
      Example(
        title: 'AsyncValue',
        icon: Icons.update,
        path: 'todo-async-value',
        builder: (_) => const async_value_example.TodoAsyncValueExample(),
        isScrollable: false,
      ),
    ],
  ),
];

void main() {
  runApp(ArticleApp());
}

class ArticleApp extends StatelessWidget {
  ArticleApp({Key? key}) : super(key: key);

  final _router = GoRouter(routes: [
    GoRoute(
      path: '/',
      builder: (_, __) => const HomePage(),
      routes: [
        for (ExampleGroup exampleTitle in examples) //
          for (Example example in exampleTitle.examples) //
            GoRoute(
              path: example.path,
              builder: (_, __) => ExampleContainer(example),
            )
      ],
    ),
  ]);

  @override
  Widget build(BuildContext context) {
    return MaterialApp.router(
      routeInformationProvider: _router.routeInformationProvider,
      routeInformationParser: _router.routeInformationParser,
      routerDelegate: _router.routerDelegate,
      title: 'Flutter par la pratique',
      theme: ThemeData(
        primarySwatch: Colors.blue,
      ),
    );
  }
}

class HomePage extends StatelessWidget {
  const HomePage({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Flutter par la pratique'),
      ),
      body: ListView.builder(
        itemBuilder: (_, index) => ExampleGroupListItem(examples[index]),
        itemCount: examples.length,
      ),
    );
  }
}

class ExampleGroupListItem extends StatelessWidget {
  const ExampleGroupListItem(this.exampleGroup, {Key? key}) : super(key: key);

  final ExampleGroup exampleGroup;

  @override
  Widget build(BuildContext context) {
    return StickyHeader(
      header: Container(
        height: 50,
        color: Colors.white,
        alignment: Alignment.centerLeft,
        padding: const EdgeInsets.symmetric(horizontal: 16),
        child: Text(
          exampleGroup.title,
          overflow: TextOverflow.ellipsis,
          style: Theme.of(context).textTheme.subtitle2,
        ),
      ),
      content: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          for (Example example in exampleGroup.examples) //
            ExampleListItem(example)
        ],
      ),
    );
  }
}

class ExampleListItem extends StatelessWidget {
  const ExampleListItem(this.example, {Key? key}) : super(key: key);

  final Example example;

  @override
  Widget build(BuildContext context) {
    return ListTile(
      leading: Icon(
        example.icon,
        color: Theme.of(context).primaryColor,
      ),
      title: Text(
        example.title,
        overflow: TextOverflow.ellipsis,
      ),
      trailing: const Icon(Icons.chevron_right),
      onTap: () => context.push('/${example.path}'),
    );
  }
}
